#include <LiquidCrystal.h>

//Resistor values:
//Voltmeter
#define VR1 4700
#define VR2 15000
//Ohmmeter
#define RR2 1000

//Pin settings
#define VOLTMETER A0
#define OHMMETER A1
#define BUTTON 2
#define BACKLIGHT 6

//Modes to toggle between
#define MODE_VOLT 0
#define MODE_RES_1K 1

//Sets initial mode
int mode = MODE_VOLT;
LiquidCrystal lcd(8,9,10,11,12,13); //Creates the lcd object

//Calculates Vin of the voltage divider
double calculateVoltage()
{
  double vout, vin, ratio;
  vout = 5 * analogRead(VOLTMETER) / (double)1024; //Turns the analogRead() measurement into a voltage
  ratio = VR1 / (double)(VR1+VR2);
  vin = vout / ratio;
  
  return vin;
}

//Calculates R1 of the voltage divider
long calculateResistance()
{
  double vout, numerator, denominator;
  vout = 5 * analogRead(OHMMETER) / (double)1024; //Turns the analogRead() measurement into a voltage
  numerator = vout * RR2;
  denominator = 5 - vout;
  if(analogRead(OHMMETER) > 1000) //Very high resistance or open circuit
    return -1;
  else
    return numerator / denominator;  
}

void switchMode() //Toggle to the next mode
{
  mode++; //Increment mode
  if(mode >= 2) //Loop mode around to the first mode
    mode = 0;
}

void setup() {
  //Set up inputs
  pinMode(VOLTMETER, INPUT);
  pinMode(OHMMETER, INPUT);
  
  lcd.begin(20,4); //Sets up the LCD as (20 characters) x (4 lines)
  attachInterrupt(digitalPinToInterrupt(BUTTON), switchMode, RISING); //Sets the button to toggle modes
}

void loop() {
  analogWrite(BACKLIGHT, 100); //Dims the LCD backlight

  //Set up LCD
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Mode:");
  lcd.setCursor(0,2);
  lcd.print("Value:");

  //Mode specific functions
  if(mode == MODE_VOLT)
  {
    lcd.setCursor(6,0);
    lcd.print("Voltage");
    lcd.setCursor(7,2);
    lcd.print(calculateVoltage()); //Print the reading
  }
  else if(mode == MODE_RES_1K)
  {
    lcd.setCursor(6,0);
    lcd.print("Resistance");
    lcd.setCursor(7,2);
    int resistance = calculateResistance(); 
    if(resistance == -1)
      lcd.print("OL"); //Print OL if the circuit is an Open Loop (resistance too high)
    else
      lcd.print(calculateResistance()); //Print the reading
  }
  
  delay(250); //Slows the loop to minimize LCD flashing
}






